﻿unit ULockToolbar;

interface

procedure Register;

implementation

uses
  Winapi.Windows, System.SysUtils, System.Classes, System.IniFiles, Vcl.Forms,
  Vcl.Menus, Vcl.ExtCtrls;

var
  FLocked: Boolean;
  FOldPopup: TNotifyEvent;
  FOldBeginBandMove: TBeginBandMoveEvent;
  FIDEMenu: TMenuItem;

function GetControlBar(out ControlBar: TControlBar): Boolean;
var
  i: Integer;
begin
  Result := False;
  if Application.MainForm = nil then Exit;
  ControlBar := TControlBar(Application.MainForm.FindComponent('ControlBar1'));
  if ControlBar = nil then
    for i := 0 to Application.MainForm.ControlCount-1 do
      if Application.MainForm.Controls[i] is TControlBar then
      begin
        ControlBar := TControlBar(Application.MainForm.Controls[i]);
        Break;
      end;
  Result := ControlBar <> nil;
end;

function GetControlBarPopup(out Popup: TPopupMenu): Boolean;
var
  controlBar: TControlBar;
begin
  Result := False;
  if not GetControlBar(controlBar) then Exit;
  Popup := controlBar.PopupMenu;
  Result := Popup <> nil;
end;

procedure UpdateToolbar;
var
  controlBar: TControlBar;
  method: TMethod;

  procedure ControlBarBeginBandMove(Self, Sender, Control: TObject; var Drag: Boolean);
  begin
    Drag := False;
  end;

begin
  if not GetControlBar(controlBar) then Exit;

  if FLocked then
  begin
    FOldBeginBandMove := controlBar.OnBeginBandMove;
    method.Code := @ControlBarBeginBandMove;
    method.Data := nil;
    controlBar.OnBeginBandMove := TBeginBandMoveEvent(method);
  end
  else begin
    controlBar.OnBeginBandMove := FOldBeginBandMove;
    FOldBeginBandMove := nil;
  end;
end;

type
  TIDEMenuItem = class(TMenuItem)
  public
    destructor Destroy; override;
  end;

destructor TIDEMenuItem.Destroy;
begin
  FIDEMenu := nil;
  inherited;
end;

procedure MenuClick(Self, Sender: TObject);
begin
  FLocked := not FLocked;
  UpdateToolbar;
end;

procedure MenuPopup(Self, Sender: TObject);
var
  method: TMethod;
begin
  if Assigned(FOldPopup) then
    FOldPopup(Sender);

  if FIDEMenu <> nil then Exit;
  FIDEMenu := TIDEMenuItem.Create(nil);
  if SameText(PreferredUILanguages, 'JA') then
    FIDEMenu.Caption := 'ツールバーをロック(&L)'
  else
    FIDEMenu.Caption := '&Lock Toolbar';
  FIDEMenu.AutoCheck := True;
  FIDEMenu.Checked := FLocked;
  method.Code := @MenuClick;
  method.Data := nil;
  FIDEMenu.OnClick := TNotifyEvent(method);
  FIDEMenu.Visible := True;
  TPopupMenu(Sender).Items.Add(FIDEMenu);
end;

procedure Register;
var
  popup: TPopupMenu;
  method: TMethod;
  ini: TIniFile;
begin
  if not GetControlBarPopup(popup) then Exit;

  FOldPopup := popup.OnPopup;
  method.Code := @MenuPopup;
  method.Data := nil;
  popup.OnPopup := TNotifyEvent(method);

  ini := TIniFile.Create(ExtractFilePath(GetModuleName(HInstance)) + 'LockToolbar.ini');
  try
    FLocked := ini.ReadBool('Settings', 'Locked', False);
    if FLocked then
      UpdateToolbar;
  finally
    ini.Free;
  end;
end;

procedure Unregister;
var
  ini: TIniFile;
  popup: TPopupMenu;
begin
  try
    ini := TIniFile.Create(ExtractFilePath(GetModuleName(HInstance)) + 'LockToolbar.ini');
    try
      ini.WriteBool('Settings', 'Locked', FLocked);
    finally
      ini.Free;
    end;
  except
  end;

  if GetControlBarPopup(popup) then
  begin
    popup.OnPopup := FOldPopup;
  end;

  if FLocked then
  begin
    FLocked := False;
    UpdateToolbar;
  end;

  FIDEMenu.Free;
end;

initialization
finalization
  Unregister;
end.